'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _transmission = require('./transmission');

var _transmission2 = _interopRequireDefault(_transmission);

var _builder = require('./builder');

var _builder2 = _interopRequireDefault(_builder);

var _events = require('events');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } // Copyright 2016 Hound Technology, Inc. All rights reserved.
// Use of this source code is governed by the Apache License 2.0
// license that can be found in the LICENSE file.

/**
 * @module
 */


var defaults = Object.freeze({
  // host to send data to
  apiHost: "https://api.honeycomb.io/",

  // sample rate of data.  causes us to send 1/sample-rate of events
  // i.e. `sampleRate: 10` means we only send 1/10th the events.
  sampleRate: 1,

  // transmission constructor, or a string "worker"/"base" to pick one of our builtin versions.
  // we fall back to the base impl if worker or a custom implementation throws on init.
  transmission: "base",

  // batch triggers
  batchSizeTrigger: 50, // we send a batch to the api when we have this many outstanding events
  batchTimeTrigger: 100, // ... or after this many ms has passed.

  // batches are sent serially (one event at a time), so we allow multiple concurrent batches
  // to increase parallelism while sending.
  maxConcurrentBatches: 10,

  // the maximum number of pending events we allow in our queue before they get batched
  pendingWorkCapacity: 10000,

  // the maximum number of s we enqueue before we drop.
  maxResponseQueueSize: 1000,

  // if this is false, all sending is disabled.  useful for disabling libhoney when testing
  disabled: false
});

/**
 * Represents a honeycomb context.  Each honeycomb context has
 * @class
 */

var Libhoney = function (_EventEmitter) {
  _inherits(Libhoney, _EventEmitter);

  /**
   * constructs a libhoney context.
   *
   * @param {Object} [opts] overrides for the defaults
   * @constructor
   * @example
   * import Libhoney from 'libhoney';
   * let honey = new Libhoney({
   *   sampleRate: 10
   * });
   */
  function Libhoney(opts) {
    _classCallCheck(this, Libhoney);

    var _this = _possibleConstructorReturn(this, (Libhoney.__proto__ || Object.getPrototypeOf(Libhoney)).call(this));

    _this._options = Object.assign({ responseCallback: _this._responseCallback.bind(_this) }, defaults, opts);
    _this._transmission = getAndInitTransmission(_this._options.transmission, _this._options);
    _this._usable = _this._transmission != null;
    _this._builder = new _builder2.default(_this);

    _this._builder.apiHost = _this._options.apiHost;
    _this._builder.writeKey = _this._options.writeKey;
    _this._builder.dataset = _this._options.dataset;
    _this._builder.sampleRate = _this._options.sampleRate;

    _this._responseQueue = [];
    return _this;
  }

  _createClass(Libhoney, [{
    key: '_responseCallback',
    value: function _responseCallback(response) {
      var queue = this._responseQueue;
      if (queue.length < this._options.maxResponseQueueSize) {
        queue.push(response);
      }
      this.emit("response", queue);
    }

    /**
     * The hostname for the Honeycomb API server to which to send events created through this libhoney
     * instance. default: https://api.honeycomb.io/
     *
     * @type {string}
     */

  }, {
    key: 'sendEvent',


    /**
     *  sendEvent takes events of the following form:
     *
     * {
     *   data: a JSON-serializable object, keys become colums in Honeycomb
     *   timestamp [optional]: time for this event, defaults to now()
     *   writeKey [optional]: your team's write key.  overrides the libhoney instance's value.
     *   dataset [optional]: the data set name.  overrides the libhoney instance's value.
     *   sampleRate [optional]: cause us to send 1 out of sampleRate events.  overrides the libhoney instance's value.
     * }
     *
     * Sampling is done based on the supplied sampleRate, so events passed to this method might not
     * actually be sent to Honeycomb.
     * @private
     */
    value: function sendEvent(event) {
      if (!this._usable) return;

      var timestamp = event.timestamp || Date.now();
      if (typeof timestamp === 'string' || typeof timestamp === 'number') timestamp = new Date(timestamp);

      if (_typeof(event.data) !== 'object' || event.data === null) {
        console.error(".data must be an object");
        return;
      }
      var postData;
      try {
        postData = JSON.stringify(event.data);
      } catch (e) {
        console.error("error converting event data to JSON: " + e);
        return;
      }

      var apiHost = event.apiHost;
      if (typeof apiHost !== 'string' || apiHost === "") {
        console.error(".apiHost must be a non-empty string");
        return;
      }

      var writeKey = event.writeKey;
      if (typeof writeKey !== 'string' || writeKey === "") {
        console.error(".writeKey must be a non-empty string");
        return;
      }

      var dataset = event.dataset;
      if (typeof dataset !== 'string' || dataset === "") {
        console.error(".dataset must be a non-empty string");
        return;
      }

      var sampleRate = event.sampleRate;
      if (typeof sampleRate !== 'number') {
        console.error(".sampleRate must be a number");
        return;
      }

      var metadata = event.metadata;

      this._transmission.sendEvent({
        timestamp: timestamp,
        apiHost: apiHost,
        postData: postData,
        writeKey: writeKey,
        dataset: dataset,
        sampleRate: sampleRate,
        metadata: metadata
      });
    }

    /**
     *  sendPresampledEvent takes events of the following form:
     *
     * {
     *   data: a JSON-serializable object, keys become colums in Honeycomb
     *   timestamp [optional]: time for this event, defaults to now()
     *   writeKey [optional]: your team's write key.  overrides the libhoney instance's value.
     *   dataset [optional]: the data set name.  overrides the libhoney instance's value.
     *   sampleRate: the rate this event has already been sampled.
     * }
     *
     * Sampling is presumed to have already been done (at the supplied sampledRate), so all events passed to this method
     * are sent to Honeycomb.
     * @private
     */

  }, {
    key: 'sendPresampledEvent',
    value: function sendPresampledEvent(event) {
      if (!this._usable) return;

      var timestamp = event.timestamp || Date.now();
      if (typeof timestamp === 'string' || typeof timestamp === 'number') timestamp = new Date(timestamp);

      if (_typeof(event.data) !== 'object' || event.data === null) {
        console.error(".data must be an object");
        return;
      }
      var postData;
      try {
        postData = JSON.stringify(event.data);
      } catch (e) {
        console.error("error converting event data to JSON: " + e);
        return;
      }

      var apiHost = event.apiHost;
      if (typeof apiHost !== 'string' || apiHost === "") {
        console.error(".apiHost must be a non-empty string");
        return;
      }

      var writeKey = event.writeKey;
      if (typeof writeKey !== 'string' || writeKey === "") {
        console.error(".writeKey must be a non-empty string");
        return;
      }

      var dataset = event.dataset;
      if (typeof dataset !== 'string' || dataset === "") {
        console.error(".dataset must be a non-empty string");
        return;
      }

      var sampleRate = event.sampleRate;
      if (typeof sampleRate !== 'number') {
        console.error(".sampleRate must be a number");
        return;
      }

      var metadata = event.metadata;

      this._transmission.sendPresampledEvent({
        timestamp: timestamp,
        apiHost: apiHost,
        postData: postData,
        writeKey: writeKey,
        dataset: dataset,
        sampleRate: sampleRate,
        metadata: metadata
      });
    }

    /**
     * adds a group of field->values to the global Builder.
     * @param {Object|Map<string, any>} data field->value mapping.
     * @returns {Libhoney} this libhoney instance.
     * @example <caption>using an object</caption>
     *   honey.add ({
     *     buildID: "a6cc38a1",
     *     env: "staging"
     *   });
     * @example <caption>using an ES2015 map</caption>
     *   let map = new Map();
     *   map.set("build_id", "a6cc38a1");
     *   map.set("env", "staging");
     *   honey.add (map);
     */

  }, {
    key: 'add',
    value: function add(data) {
      this._builder.add(data);
      return this;
    }

    /**
     * adds a single field->value mapping to the global Builder.
     * @param {string} name name of field to add.
     * @param {any} val value of field to add.
     * @returns {Libhoney} this libhoney instance.
     * @example
     *   honey.addField("build_id", "a6cc38a1");
     */

  }, {
    key: 'addField',
    value: function addField(name, val) {
      this._builder.addField(name, val);
      return this;
    }

    /**
     * adds a single field->dynamic value function to the global Builder.
     * @param {string} name name of field to add.
     * @param {function(): any} fn function that will be called to generate the value whenever an event is created.
     * @returns {Libhoney} this libhoney instance.
     * @example
     *   honey.addDynamicField("process_heapUsed", () => process.memoryUsage().heapUsed);
     */

  }, {
    key: 'addDynamicField',
    value: function addDynamicField(name, fn) {
      this._builder.addDynamicField(name, fn);
      return this;
    }

    /**
     * creates and sends an event, including all global builder fields/dyn_fields, as well as anything in the optional data parameter.
     * @param {Object|Map<string, any>} data field->value mapping.
     * @example <caption>using an object</caption>
     *   honey.sendNow ({
     *     responseTime_ms: 100,
     *     httpStatusCode: 200
     *   });
     * @example <caption>using an ES2015 map</caption>
     *   let map = new Map();
     *   map.set("responseTime_ms", 100);
     *   map.set("httpStatusCode", 200);
     *   honey.sendNow (map);
     */

  }, {
    key: 'sendNow',
    value: function sendNow(data) {
      return this._builder.sendNow(data);
    }

    /**
     * creates and returns a new Event containing all fields/dyn_fields from the global Builder, that can be further fleshed out and sent on its own.
     * @returns {Event} an Event instance
     * @example <caption>adding data at send-time</caption>
     *   let ev = honey.newEvent();
     *   ev.addField("additionalField", value);
     *   ev.send();
     */

  }, {
    key: 'newEvent',
    value: function newEvent() {
      return this._builder.newEvent();
    }

    /**
     * creates and returns a clone of the global Builder, merged with fields and dyn_fields passed as arguments.
     * @param {Object|Map<string, any>} fields a field->value mapping to merge into the new builder.
     * @param {Object|Map<string, any>} dyn_fields a field->dynamic function mapping to merge into the new builder.
     * @returns {Builder} a Builder instance
     * @example <caption>no additional fields/dyn_field</caption>
     *   let builder = honey.newBuilder();
     * @example <caption>additional fields/dyn_field</caption>
     *   let builder = honey.newBuilder({ requestId },
     *                                  {
     *                                    process_heapUsed: () => process.memoryUsage().heapUsed
     *                                  });
     */

  }, {
    key: 'newBuilder',
    value: function newBuilder(fields, dyn_fields) {
      return this._builder.newBuilder(fields, dyn_fields);
    }
  }, {
    key: 'apiHost',
    set: function set(v) {
      this._builder.apiHost = v;
    }
    /**
     * The hostname for the Honeycomb API server to which to send events created through this libhoney
     * instance. default: https://api.honeycomb.io/
     *
     * @type {string}
     */
    ,
    get: function get() {
      return this._builder.apiHost;
    }

    /**
     * The Honeycomb authentication token. If it is set on a libhoney instance it will be used as the
     * default write key for all events. If absent, it must be explicitly set on a Builder or
     * Event. Find your team write key at https://ui.honeycomb.io/account
     *
     * @type {string}
     */

  }, {
    key: 'writeKey',
    set: function set(v) {
      this._builder.writeKey = v;
    }
    /**
     * The Honeycomb authentication token. If it is set on a libhoney instance it will be used as the
     * default write key for all events. If absent, it must be explicitly set on a Builder or
     * Event. Find your team write key at https://ui.honeycomb.io/account
     *
     * @type {string}
     */
    ,
    get: function get() {
      return this._builder.writeKey;
    }

    /**
     * The name of the Honeycomb dataset to which to send events through this libhoney instance.  If
     * it is specified during libhoney initialization, it will be used as the default dataset for all
     * events. If absent, dataset must be explicitly set on a builder or event.
     *
     * @type {string}
     */

  }, {
    key: 'dataset',
    set: function set(v) {
      this._builder.dataset = v;
    }
    /**
     * The name of the Honeycomb dataset to which to send these events through this libhoney instance.
     * If it is specified during libhoney initialization, it will be used as the default dataset for
     * all events. If absent, dataset must be explicitly set on a builder or event.
     *
     * @type {string}
     */
    ,
    get: function get() {
      return this._builder.dataset;
    }

    /**
     * The rate at which to sample events. Default is 1, meaning no sampling. If you want to send one
     * event out of every 250 times send() is called, you would specify 250 here.
     *
     * @type {number}
     */

  }, {
    key: 'sampleRate',
    set: function set(v) {
      this._builder.sampleRate = v;
    }
    /**
     * The rate at which to sample events. Default is 1, meaning no sampling. If you want to send one
     * event out of every 250 times send() is called, you would specify 250 here.
     *
     * @type {number}
     */
    ,
    get: function get() {
      return this._builder.sampleRate;
    }
  }]);

  return Libhoney;
}(_events.EventEmitter);

exports.default = Libhoney;


function getAndInitTransmission(transmission, options) {
  if (options.disabled) {
    return null;
  }

  if (typeof transmission === "string") {
    if (transmission === "base") {
      transmission = _transmission2.default;
    } else if (transmission === "worker") {
      console.warn("worker implementation not ready yet.  using base implementation");
      transmission = _transmission2.default;
    } else {
      throw new Error('unknown transmission implementation "' + transmission + '".');
    }
  } else if (typeof transmission !== "function") {
    throw new Error(".transmission must be one of 'base'/'worker' or a constructor.");
  }

  try {
    return new transmission(options);
  } catch (e) {
    if (transmission == _transmission2.default) {
      throw new Error("unable to initialize base transmission implementation.", e);
    }

    console.warn("failed to initialize transmission, falling back to base implementation.");
    try {
      transmission = new _transmission2.default(options);
    } catch (e) {
      throw new Error("unable to initialize base transmission implementation.", e);
    }
  }

  return transmission;
}